#include "mpi.h"
#include <stdio.h>
#include <stdlib.h>

#define NEIGHBOUR_COUNT 2

int main(int argc, char *argv[]) {
	int errs = 0, i, k;
	int size, topo_type;
	int *degreeIndices, *edges, *dupDegreeIndices, *dupEdges;
	MPI_Comm comm1, comm2;

	MPI_Init( &argc, &argv );
	MPI_Comm_size( MPI_COMM_WORLD, &size );

	// Napravi graf, dohvati njegove karakteristike, dupliraj ga 
	// i proveri da li novi komunikator ima iste karakeristike
	if (size >= 3) {
		// po jedan član niza indeksa, za svaki proces
		degreeIndices = (int*)malloc(size * sizeof(int) );
		// svaki proces ima dva suseda, otud dvostruka veličina prethodnog niza
		edges = (int*)malloc(size * NEIGHBOUR_COUNT * sizeof(int) );
		// ako alokacija ne uspe, kraj sveta
		if (!degreeIndices || !edges) {
			printf( "Unable to allocate %d words for degreeIndices or edges\n", 3 * size );fflush(stdout);
			MPI_Abort( MPI_COMM_WORLD, 1 );
		}
		// stepen svakog čvora određen je formulom iz standarda
		degreeIndices[0] = NEIGHBOUR_COUNT;
		for (i=1; i<size; i++) {
			degreeIndices[i] = NEIGHBOUR_COUNT + degreeIndices[i-1];
		}
		// svaki čvor ima dva suseda
		for (k=0, i=0; i<size; i++) {
			edges[k++] = (i-1+size) % size;
			edges[k++] = (i+1) % size;
		}
		// napravi graf opisanih karakteristika
		MPI_Graph_create( MPI_COMM_WORLD, size, degreeIndices, edges, 0, &comm1 );

		// dupliraj taj graf
		MPI_Comm_dup( comm1, &comm2 );

		// proveri da li je stvarno graf
		MPI_Topo_test( comm2, &topo_type );
		if (topo_type != MPI_GRAPH) {
			errs++;
			printf( "Topo type of duped graph was not graph\n" );fflush(stdout);
		}
		// ako jeste, 
		else {
			int dupNodes, dupEdgesCount;
			MPI_Graphdims_get( comm2, &dupNodes, &dupEdgesCount );
			if (dupNodes != size) {
				errs++;
				printf( "dupNodes = %d, should be %d\n", dupNodes, size );fflush(stdout);
			}
			if (dupEdgesCount != 2*size) {
				errs++;
				printf( "dupEdgesCount = %d, should be %d\n", dupEdgesCount, NEIGHBOUR_COUNT*size );fflush(stdout);
			}
			dupDegreeIndices = (int*)malloc(size * sizeof(int) );
			dupEdges = (int*)malloc(size * NEIGHBOUR_COUNT * sizeof(int) );
			if (!dupDegreeIndices || !dupEdges) {
				printf( "Unable to allocate total of %d words for dupDegreeIndices and dupEdges\n", 3 * size );fflush(stdout);
				MPI_Abort( MPI_COMM_WORLD, 1 );
			}

			MPI_Graph_get( comm2, size, 2*size, dupDegreeIndices, dupEdges );
			for (i=0; i<size; i++) {
				if (degreeIndices[i] != dupDegreeIndices[i]) {
					printf( "%d = degreeIndices[%d] != dupDegreeIndices[%d] = %d\n", degreeIndices[i], i, i, dupDegreeIndices[i] );fflush(stdout);
					errs++;
				}
			}
			for (i=0; i<NEIGHBOUR_COUNT*size; i++) {
				if (edges[i] != dupEdges[i]) {
					printf( "%d = edges[%d] != dupEdges[%d] = %d\n", edges[i], i, i, dupEdges[i] );fflush(stdout);
					errs++;
				}
			}
			free( dupDegreeIndices );
			free( dupEdges );
		}
		free( degreeIndices );
		free( edges );
		MPI_Comm_free( &comm2 );
		MPI_Comm_free( &comm1 );
	}

	MPI_Finalize();
	return 0;
}
